#!/usr/bin/python

import getopt
import logging
import re
import string
import sys

# suppress the no route warning in scapy when loading
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

# import scapy
from scapy.all import *

def usage():
    """
    Function for presenting usage of the tool.
    """
    print "CDP Parse by Carlos Perez carlos_perez@darkoperator.com"
    print "Tool for printing to STDOUT information on CDP packets found capture"
    print "file. Will print all supported options.\n"
    print "cdp_parser.py <OPTIONS>"
    print "-F <dir>  Directory containing pcaps."
    print "-f <pcap> pcap file."
    sys.exit(0)

def process_packets(pkts):
    """
    Function for processing packets and printing information of CDP Packets
    """

    for p in pkts:
        # Check if the packet is a CDP Packet
        if Dot3 in p and p.dst == '01:00:0c:cc:cc:cc':

            print "\n*******************************"
            print "Source MAC:", p.src
            # Process each field in the packet message
            for f in p[CDPv2_HDR].fields["msg"]:

                # Check if the filed type is a known one
                if f.type in _cdp_tlv_types:

                    # Process each field according to type
                    f_type = _cdp_tlv_types[f.type]

                    # Make sure we process each address in the message
                    if re.match(r"(Addresses|Management Address)", f_type):
                        for ip in f.fields["addr"]:
                            print f_type, ip.addr

                    elif f_type == "Software Version":
                        print f_type+":"
                        print "\t" + string.replace(f.val, "\n", "\n\t")

                    elif f_type == "Port ID":
                        print f_type, ":", f.iface

                    elif f_type == "Capabilities":
                        # Ugly but works :)
                        print f_type, ":", "".join(re.findall(r"cap\s*=(\S*)", str(f.show)))

                    elif re.match(r"Native VLAN|VoIP VLAN Reply",f_type):
                        print f_type, ":", f.vlan

                    elif f_type == "Duplex":
                        print f_type, ":", _cdp_duplex[f.duplex]

                    elif f_type == "IP Prefix":
                        print f_type, ":", f.defaultgw

                    elif f_type == "Power":
                        print f_type, ":", f.power, " mW"

                    # Fields not yet implemented in the current version of the
                    # contributed cdp module.
                    elif f_type == "Power Available":
                        # I know, this should provide the amount of power
                        print f_type, ": POE Enabled"

                    elif f_type == "Protocol Hello":
                        pass

                    else:
                        try:
                            # Make sure we do not have an empty value and print
                            if f.val is not '\0' and len(f.val) != 0: print f_type, ":", f.val

                        except Exception, e:
                            print "ERROR!!!!:", f_type
                            print e
                            print "Send error to: carlos_perez[at]darkoperator.com"
                            pass
def main():

    try:
        # Check version
        if not re.match(r"2\.[2-9]\.\S*", config.conf.version):
            print "You are not running the latest scapy release."
            print "Please go to http://trac.secdev.org/scapy and follow the"
            print "the instructions to download the latest versions."
            sys.exit(1)

        # load the support for CDP Packets
        load_contrib("cdp")

        # Set Variables for Options
        folder = None
        pcap_file = None
        pcap_files = []

        # Check that options are given
        if len(sys.argv) == 1:
            usage()

        # Set Options
        options, remainder = getopt.getopt(sys.argv[1:], 'F:f:h')

        # Parse Options
        for opt, arg in options:
            if opt in ('-F'):
                folder = arg
            elif opt in ('-f'):
                pcap_file = arg
            elif opt in ('-h'):
                usage()
            else:
                usage()

        # Process folder with pcap files
        if folder:
            if os.path.isdir(folder):
                for item in os.listdir(arg):
                    fullpath = os.path.join(arg, item)
                    if os.path.isfile(fullpath) and ('.cap' in item or '.pcap' in item or '.dump' in item):
                        pcap_files.append(fullpath)
            else:
                print "ERROR:", folder, "does not exists!"
                sys.exit(1)

        # Process single pcap file
        if pcap_file:
            if os.path.isfile(pcap_file):
                pcap_files.append(pcap_file)
            else:
                print "ERROR:",pcap_file,"does not exist!"
                sys.exit(1)

        # Process all files selected and extract CDP Info
        for f in pcap_files:
            print "Processing: ",f
            pcap = rdpcap(f)
            process_packets(pcap)
    except Exception, e:
        print e
        print "Send error to: carlos_perez[at]darkoperator.com"
        pass

if __name__ == '__main__':
    main()